import _Load from './core';
import { isArray, isString, isFunction, isObject, distinct } from '../util';

const CONFIG = {};

const Load = function (urls, resolve) {
    resolve = resolve || function () { };
    let styles = (isArray(urls) ? urls : [urls]).filter((url) => {
        return url && url.type === 'css';
    }).map((u) => {
        return u.link;
    });
    if (!CONFIG.enableRepeat) {
        styles = distinct(styles).filter((s) => { return !checkExist(s); });
    }
    if (styles.length > 0) {
        styles.forEach((s) => { Load.rss.push(s); });
        styles = styles.reverse();
    }
    let jss = (isArray(urls) ? urls : [urls]).filter((url) => {
        return url && url.type === 'js';
    }).map((u) => {
        return u.link;
    });
    if (!CONFIG.enableRepeat) {
        jss = distinct(jss).filter((j) => { return !checkExist(j); });
    }
    if (jss.length > 0) {
        jss.forEach((j) => { Load.rss.push(j); });
    } else if (!jss || isFunction(resolve)) {
        resolve();
    }
    if (styles.length > 0) {
        _useCSS(styles, () => {
            if (jss.length > 0) {
                _useJS(jss, resolve);
            } else if (isFunction(resolve)) {
                resolve();
            }
        });
    } else if (jss.length) {
        _useJS(jss, resolve);
    }
};

Load.config = ({ path = '', enableRepeat = false, timeout = 60000, position = 'head' }) => {
    CONFIG.path = path;
    CONFIG.enableRepeat = enableRepeat;
    CONFIG.timeout = timeout;
    CONFIG.position = position;
};

Load.rss = [];

/**
 * 获取资源路径
 * @param {String} url 路径地址
 * @param {Boolean} prefix 是否只返回前缀路径
 * @return {String} 路径
 */
function getPath(url, prefix) {
    let base = '';
    const temp = url.split(':');

    if (temp.length === 1) {
        if (isString(CONFIG.path)) {
            return prefix ? CONFIG.path : CONFIG.path + url;
        }
        if (isObject(CONFIG.path)) {
            base = CONFIG.path.main || '';

            return prefix ? base : base + url;
        }
    }
    if (temp.length === 2 && temp[0] !== 'http' && temp[0] !== 'https') {
        if (isString(CONFIG.path)) {
            base = CONFIG.path;
        }
        if (isObject(CONFIG.path) && CONFIG.path[temp[0]]) {
            base = CONFIG.path[temp[0]];
        }

        return prefix ? base : base + temp[1];
    }

    return base;
}

function getCSSPath(url) {
    if (url.indexOf('http://') === 0 || url.indexOf('https://') === 0) {
        return url;
    }
    let base = '';
    const temp = url.split(':');

    if (temp.length === 1) {
        if (isString(CONFIG.path)) {
            return CONFIG.path + url;
        }
        if (isObject(CONFIG.path)) {
            base = CONFIG.path.main || '';

            return base + url;
        }
    }
    if (temp.length === 2) {
        if (isString(CONFIG.path)) {
            base = CONFIG.path;
        }
        if (isObject(CONFIG.path) && CONFIG.path[temp[0]]) {
            base = CONFIG.path[temp[0]];
        }

        return base + temp[1];
    }

    return base;
}

function getJSPathPrefix(url) {
    let base = '';
    const temp = url.split(':');

    if (temp.length === 1) {
        if (isString(CONFIG.path)) {
            return CONFIG.path;
        }
        if (isObject(CONFIG.path)) {
            return CONFIG.path.main || '';
        }
    }
    if (temp.length === 2 && temp[0] !== 'http' && temp[1] !== 'https') {
        if (isString(CONFIG.path)) {
            base = CONFIG.path;
        }
        if (isObject(CONFIG.path) && CONFIG.path[temp[0]]) {
            base = CONFIG.path[temp[0]];
        }
    }

    return base;
}

function getJSPathSuffix(url) {
    const temp = url.split(':');

    if (temp.length === 1) {
        return url;
    }
    if (temp.length === 2) {
        return temp[1];
    }

    return url;
}

function poll(node, callback) {
    if (callback.isCalled) {
        return;
    }
    let isLoaded = false;
    if (node.sheet) {
        isLoaded = true;
    }
    if (isLoaded) {
        setTimeout(() => {
            callback();
        }, 1);
    } else {
        setTimeout(() => {
            poll(node, callback);
        }, 1);
    }
}

function styleOnload(node, callback) {
    // for IE6-9 and Opera
    if (node.attachEvent) {
        node.attachEvent('onload', callback);
    } else {
        setTimeout(() => {
            poll(node, callback);
        }, 0);
    }
}

function loadCSS(_url, callback) {
    if (!_url || !isString(_url) || _url.indexOf('.css') < 0) return null;
    const start = _url.lastIndexOf('/');
    const end = _url.lastIndexOf('.css');
    const name = _url.substring(start + 1, end);
    const url = getCSSPath(_url);
    const links = document.getElementsByTagName('link');
    const link = document.createElement('link');
    const head = document.getElementsByTagName('head')[0];

    for (let i = 0; i < links.length; i++) {
        if (links[i].href.indexOf(url) > -1) {
            return null;
        }
    }

    link.type = 'text/css';
    link.rel = 'stylesheet';
    link.href = url;
    link.setAttribute('data-mod', name);
    link.setAttribute('data-from', 'PL');

    head.insertBefore(link, head.getElementsByTagName('link')[0] || null);

    return link;
}

let cssTime = 0;

function _useCSS(urls, callback) {
    if (urls) {
        const loadeds = [];
        const result = isArray(urls) ? urls : [urls];

        (result).forEach((url) => {
            if (!isFunction(callback)) {
                loadCSS(url);
            } else {
                styleOnload(loadCSS(url), () => {
                    loadeds.push(true);
                });
            }
        })
        if (isFunction(callback)) {
            cssTime = setInterval(() => {
                if (loadeds.length === result.length || cssTime >= CONFIG.timeout) {
                    clearInterval(cssTime);
                    if (callback && isFunction(callback)) {
                        callback();
                    }
                }
            }, 1);
        }
    }
}

function _useJS(urls, callback) {
    if (!urls) return;
    if (isArray(urls)) {
        if (urls.length > 1) {
            if (isString(urls[0])) {
                _Load.setOptions({ BasePath: getJSPathPrefix(urls[0]), position: CONFIG.position }).script(getJSPathSuffix(urls[0])).wait(() => {
                    _useJS(urls.slice(1), callback);
                });
            }
        } else if (urls.length === 1) {
            _Load.setOptions({ BasePath: getJSPathPrefix(urls[0]), position: CONFIG.position }).script(getJSPathSuffix(urls[0])).wait(callback);
        }
    } else if (isString(urls)) {
        _Load.setOptions({ BasePath: getJSPathPrefix(urls), position: CONFIG.position }).script(getJSPathSuffix(urls)).wait(callback);
    }
}

function checkExist(url) {
    return Load.rss.filter((rs) => { return rs === url; }).length === 1;
}

Load.use = (urls, callback) => {
    let styles = (isArray(urls) ? urls : [urls]).filter((url) => {
        return url && isString(url) && url.substring(url.length - 4) === '.css';
    });
    if (!CONFIG.enableRepeat) {
        styles = distinct(styles).filter((s) => { return !checkExist(s); });
    }
    if (styles.length > 0) {
        styles.forEach((s) => { Load.rss.push(s); });
        styles = styles.reverse();
    }
    let jss = (isArray(urls) ? urls : [urls]).filter((url) => {
        return url && isString(url) && url.substring(url.length - 3) === '.js';
    });
    if (!CONFIG.enableRepeat) {
        jss = distinct(jss).filter((j) => { return !checkExist(j); });
    }
    if (jss.length > 0) {
        jss.forEach((j) => { Load.rss.push(j); });
    }
    if (styles.length > 0) {
        _useCSS(styles, () => {
            if (jss.length > 0) {
                _useJS(jss, callback);
            } else if (isFunction(callback)) {
                callback();
            }
        });
    } else if (jss.length) {
        _useJS(jss, callback);
    }
};

export default Load;